import base64
import hashlib
import hmac
import os
import shutil
from datetime import datetime
from time import mktime
from urllib.parse import urlencode
from wsgiref.handlers import format_date_time

from aiges.client.utils.exception import FileNotFoundException, AssembleHeaderException
from urllib import parse


def get_file_bytes(fd):
    """
    根据文件路径（一般相对）获取二进制数据
    :param fd: 文件路径
    :return: 二进制数据，不存在时为 None供后续判断
    """
    if os.path.exists(fd):
        with open(fd, "rb") as f:
            f_data = f.read()
        f.close()
    else:
        raise FileNotFoundException("{}：文件不存在".format(fd))
    return f_data


def del_file(filepath):
    """
    删除某一目录下的所有文件或文件夹
    :param filepath: 路径
    :return:
    """
    del_list = os.listdir(filepath)
    for f in del_list:
        file_path = os.path.join(filepath, f)
        if os.path.isfile(file_path):
            os.remove(file_path)
        elif os.path.isdir(file_path):
            shutil.rmtree(file_path)


# 生成鉴权的url
def build_auth_request_url(request_url, method="POST", api_key="", api_secret=""):
    url_result = parse.urlparse(request_url)
    date = format_date_time(mktime(datetime.now().timetuple()))
    signature_origin = "host: {}\ndate: {}\n{} {} HTTP/1.1".format(url_result.hostname, date, method, url_result.path)
    signature_sha = hmac.new(api_secret.encode('utf-8'), signature_origin.encode('utf-8'),
                             digestmod=hashlib.sha256).digest()
    signature_sha = base64.b64encode(signature_sha).decode(encoding='utf-8')
    authorization_origin = "api_key=\"%s\", algorithm=\"%s\", headers=\"%s\", signature=\"%s\"" % (
        api_key, "hmac-sha256", "host date request-line", signature_sha)
    authorization = base64.b64encode(authorization_origin.encode('utf-8')).decode(encoding='utf-8')
    values = {
        "host": url_result.hostname,
        "date": date,
        "authorization": authorization
    }
    return request_url + "?" + urlencode(values)



#!/usr/bin/env python
# coding:utf-8
""" 
@author: nivic ybyang7
@license: Apache Licence 
@file: client
@time: 2022/11/13
@contact: ybyang7@iflytek.com
@site:  
@software: PyCharm 

# code is far away from bugs with the god animal protecting
    I love animals. They taste delicious.
              ┏┓      ┏┓
            ┏┛┻━━━┛┻┓
            ┃      ☃      ┃
            ┃  ┳┛  ┗┳  ┃
            ┃      ┻      ┃
            ┗━┓      ┏━┛
                ┃      ┗━━━┓
                ┃  神兽保佑    ┣┓
                ┃　永无BUG！   ┏┛
                ┗┓┓┏━┳┓┏┛
                  ┃┫┫  ┃┫┫
                  ┗┻┛  ┗┻┛ 
"""

#  Copyright (c) 2022. Lorem ipsum dolor sit amet, consectetur adipiscing elit.
#  Morbi non lorem porttitor neque feugiat blandit. Ut vitae ipsum eget quam lacinia accumsan.
#  Etiam sed turpis ac ipsum condimentum fringilla. Maecenas magna.
#  Proin dapibus sapien vel ante. Aliquam erat volutpat. Pellentesque sagittis ligula eget metus.
#  Vestibulum commodo. Ut rhoncus gravida arcu.


from aiges.client.utils.aipass_client import execute
from aiges.client.utils.ne_utils import build_auth_request_url, get_file_bytes
from aiges.utils.log import log
import base64
import copy
from urllib import parse
import requests
import json
import jsonpath_rw
import os

media_type_list = ["text", "audio", "image", "video"]

config_dict = [
    {
        "model_name": "VOICE2VEC",
        "api_key": "",
        "api_secret": "",
        "app_id": "",
        "method": ""
    }
]




class Client(object):
    def __init__(self, model, api_key, api_secret, app_id, method="POST"):
        self.init_model(model)
        self.api_key = api_key
        self.api_secret = api_secret
        self.app_id = app_id
        self.method = method
        pass

    def init_model(self, model):
        if model not in models:
            raise NoSuchModelError("no such this model: %s" % model)
        self.input_tpl = InputData(model)

        # 初始化client对象 schema输入
        if model == VOICE2VEC:
            self.api_url = VOICE2VEC_URL
            self.res_path_list = ['$..payload.fea_image', '$..payload.fea', '$..payload.fbank_image', ]

        elif model == AV2VEC:
            self.api_url = AV2VEC_URL
            self.res_path_list = ['$..payload.fea_image', '$..payload.feature', ]

    def execute(self, **kwargs):
        request_url = self.api_url
        app_id = self.app_id
        api_secret = self.api_secret
        api_key = self.api_key
        # 获取请求url
        auth_request_url = build_auth_request_url(request_url, self.method, api_key, api_secret)

        url_result = parse.urlparse(request_url)
        headers = {'content-type': "application/json", 'host': url_result.hostname, 'app_id': app_id}
        request_data = self.input_tpl.prepare(**kwargs)
        # 程序启动的时候设置APPID
        request_data['header']['app_id'] = self.app_id
        # 准备待发送的数据
        new_request_data = self.prepare_req_data(request_data)

        # execute(self.api_url, request_data, "POST", APPId, APIKey, APISecret)
        response = requests.post(auth_request_url, data=json.dumps(new_request_data), headers=headers)
        return self.do_response(response)

    # 准备请求数据
    def prepare_req_data(self, request_data):
        new_request_data = copy.deepcopy(request_data)
        media_path2name = {}
        for media_type in media_type_list:
            media_expr = jsonpath_rw.parse("$..payload.*.{}".format(media_type))
            media_match = media_expr.find(new_request_data)
            if len(media_match) > 0:
                for media in media_match:
                    media_path2name[str(media.full_path)] = media.value
        for media_path, media_name in media_path2name.items():
            payload_path_list = media_path.split(".")
            f_data = get_file_bytes(media_name)
            new_request_data['header']['status'] = 3
            new_request_data['payload'][payload_path_list[1]][payload_path_list[2]] = base64.b64encode(f_data).decode()
            new_request_data['payload'][payload_path_list[1]]['status'] = 3
        return new_request_data

    def do_response(self, response):
        res = {"code": 0, "msg": "", "data": None}
        temp_result = json.loads(response.content.decode())
        log.debug("响应数据:{}\n".format(temp_result))
        header = temp_result.get('header')
        if header is None:
            res["code"] = response.status_code
            msg = "header为空:%s " % response.content
            if response.status_code == 401:
                msg += "\n请检查授权apikey,secret,app_id配置"
            res["msg"] = msg
            return res

        code = header.get('code')
        if header is None or code != 0:
            msg = "获取结果失败，请根据code查证问题原因, code: %s" % code
            log.error(msg)
            res["code"] = -1
            res["msg"] = msg
            return res
        log.info("sid:{}".format(header.get('sid')))
        data = {}
        for response_path in self.res_path_list:
            response_expr = jsonpath_rw.parse(response_path)
            response_match = response_expr.find(temp_result)
            if len(response_match) > 0:
                for response_item in response_match:
                    if response_item.value is None:
                        continue
                    encoding = response_item.value.get('encoding')
                    if encoding is None or len(encoding) == 0:
                        continue
                    for media_type in media_type_list:
                        media_value = response_item.value.get(media_type)
                        if media_value is None or len(media_value) == 0:
                            continue
                        real_data = base64.b64decode(media_value)
                        response_path_split_list = response_path.split('.')
                        data[response_path_split_list[-1]] = real_data
        return data


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: aiges/aiges_inner/aiges_inner.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#aiges/aiges_inner/aiges_inner.proto\x12\x05\x61iges\"\x12\n\x03Ret\x12\x0b\n\x03ret\x18\x01 \x01(\x05\"\x1a\n\x05SvcId\x12\x11\n\tserviceId\x18\x01 \x01(\t\"\x16\n\x06Schema\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t\"\xac\x01\n\x0bRequestData\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x0b\n\x03len\x18\x02 \x01(\x04\x12\x0c\n\x04type\x18\x03 \x01(\r\x12\x0e\n\x06status\x18\x04 \x01(\r\x12*\n\x04\x64\x65sc\x18\x05 \x03(\x0b\x32\x1c.aiges.RequestData.DescEntry\x12\x0c\n\x04\x64\x61ta\x18\x06 \x01(\x0c\x1a+\n\tDescEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xae\x01\n\x0cResponseData\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x0b\n\x03len\x18\x02 \x01(\x04\x12\x0c\n\x04type\x18\x03 \x01(\r\x12\x0e\n\x06status\x18\x04 \x01(\r\x12+\n\x04\x64\x65sc\x18\x05 \x03(\x0b\x32\x1d.aiges.ResponseData.DescEntry\x12\x0c\n\x04\x64\x61ta\x18\x06 \x01(\x0c\x1a+\n\tDescEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"G\n\x08Response\x12\x0b\n\x03tag\x18\x01 \x01(\t\x12\x0b\n\x03ret\x18\x02 \x01(\x05\x12!\n\x04list\x18\x03 \x03(\x0b\x32\x13.aiges.ResponseData\"l\n\x0bInitRequest\x12.\n\x06\x63onfig\x18\x01 \x03(\x0b\x32\x1e.aiges.InitRequest.ConfigEntry\x1a-\n\x0b\x43onfigEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x93\x01\n\x07Request\x12\x0b\n\x03tag\x18\x01 \x01(\t\x12*\n\x06params\x18\x02 \x03(\x0b\x32\x1a.aiges.Request.ParamsEntry\x12 \n\x04list\x18\x03 \x03(\x0b\x32\x12.aiges.RequestData\x1a-\n\x0bParamsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x1d\n\rStreamRequest\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t2\x98\x02\n\x0eWrapperService\x12/\n\x0bwrapperInit\x12\x12.aiges.InitRequest\x1a\n.aiges.Ret\"\x00\x12\x34\n\x0fwrapperOnceExec\x12\x0e.aiges.Request\x1a\x0f.aiges.Response\"\x00\x12.\n\rwrapperSchema\x12\x0c.aiges.SvcId\x1a\r.aiges.Schema\"\x00\x12\x39\n\ntestStream\x12\x14.aiges.StreamRequest\x1a\x0f.aiges.Response\"\x00(\x01\x30\x01\x12\x34\n\x0b\x63ommunicate\x12\x0e.aiges.Request\x1a\x0f.aiges.Response\"\x00(\x01\x30\x01\x42\tZ\x07./protob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'aiges.aiges_inner.aiges_inner_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\007./proto'
  _REQUESTDATA_DESCENTRY._options = None
  _REQUESTDATA_DESCENTRY._serialized_options = b'8\001'
  _RESPONSEDATA_DESCENTRY._options = None
  _RESPONSEDATA_DESCENTRY._serialized_options = b'8\001'
  _INITREQUEST_CONFIGENTRY._options = None
  _INITREQUEST_CONFIGENTRY._serialized_options = b'8\001'
  _REQUEST_PARAMSENTRY._options = None
  _REQUEST_PARAMSENTRY._serialized_options = b'8\001'
  _RET._serialized_start=46
  _RET._serialized_end=64
  _SVCID._serialized_start=66
  _SVCID._serialized_end=92
  _SCHEMA._serialized_start=94
  _SCHEMA._serialized_end=116
  _REQUESTDATA._serialized_start=119
  _REQUESTDATA._serialized_end=291
  _REQUESTDATA_DESCENTRY._serialized_start=248
  _REQUESTDATA_DESCENTRY._serialized_end=291
  _RESPONSEDATA._serialized_start=294
  _RESPONSEDATA._serialized_end=468
  _RESPONSEDATA_DESCENTRY._serialized_start=248
  _RESPONSEDATA_DESCENTRY._serialized_end=291
  _RESPONSE._serialized_start=470
  _RESPONSE._serialized_end=541
  _INITREQUEST._serialized_start=543
  _INITREQUEST._serialized_end=651
  _INITREQUEST_CONFIGENTRY._serialized_start=606
  _INITREQUEST_CONFIGENTRY._serialized_end=651
  _REQUEST._serialized_start=654
  _REQUEST._serialized_end=801
  _REQUEST_PARAMSENTRY._serialized_start=756
  _REQUEST_PARAMSENTRY._serialized_end=801
  _STREAMREQUEST._serialized_start=803
  _STREAMREQUEST._serialized_end=832
  _WRAPPERSERVICE._serialized_start=835
  _WRAPPERSERVICE._serialized_end=1115
# @@protoc_insertion_point(module_scope)
